--	Fire.lua - Class representing a flame.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

class 'Fire' (Block)

--------------------------------------------------------------------------------
function Fire:__init(pos, col)
	Block.__init(self, pos, col)

	self.colour = hslToRgb(ThreeFloats(0.09, 0.7, 0.8))
	self.newColour = ThreeFloats(self.colour)
	self.typeName = "Fire"

	--If this is less than 1, object will start shaking, then melt when it
	--reaches 1.
	self.shake = 1.0
	self.shakeIndex = 0.0
	self.wobbleIndex = 0.0

	--The flame animation frames.
	self.flameImages = {"Fire",
						"Fire1",
						"Fire2",
						"Fire1"}
	--Used to animate the flames.
	self.animIndex = 1
	self.animCount = 8

	particles:triggerSmoke(self.position, ThreeFloats(0.8))
end

--------------------------------------------------------------------------------
--	Updates the block.
function Fire:update(app)
	--Check if we've got any neighbours which need a shake.
	for i=1,#self.neighbours do
		if (self.neighbours[i].typeName == "Flammable") or
		   (self.neighbours[i].typeName == "Exploding") or
		   (self.neighbours[i].typeName == "Expando") or
		   (self.neighbours[i].typeName == "Ice") then
			if (self.neighbours[i].shake >= 1.0) then
				self.neighbours[i].shake = 0.0
			end

			if (self.neighbours[i].typeName == "Expando") then
				self.shake = 0.0
			end

			break
		end
	end

	--If we're not shaking, check our neighbours to see if we should be.
	if (self.shake >= 1.0) then
		for i=1,#self.neighbours do
			if (self.neighbours[i].typeName == "Water") then
				self.shake = 0.0
				break
			end
		end
	else
		self.shake = self.shake + 0.02
		self.shakeIndex = self.shakeIndex + 0.7
		if (self.shakeIndex > (math.pi * 2.0)) then
			self.shakeIndex = 0.0
		end

		self.wobbleIndex = self.wobbleIndex + 0.8
		if (self.wobbleIndex > (math.pi * 2.0)) then
			self.wobbleIndex = 0.0
		end

		if (self.shake >= 1.0) then
			self.canDelete = true
			particles:triggerSmoke(self.position, ThreeFloats(0.8))

			soundsQueue[#soundsQueue+1] = "FirePutOut"
		end
	end

	--Update the animation.
	self.animCount = self.animCount - 1
	if (self.animCount < 1) then
		self.animCount = 6

		self.animIndex = self.animIndex + 1
		if (self.animIndex > #self.flameImages) then
			self.animIndex = 1
		end
	end
end

--------------------------------------------------------------------------------
--	Draws the block.
function Fire:draw(d)
	tempPos = TwoFloats(self.position-24.0)
	tempScale = TwoFloats(1.0)

	if (self.shake < 1.0) then
		tempScale.y = 1.0 + (math.sin(self.wobbleIndex) * 0.2 * self.shake)
		tempPos.x = tempPos.x + (math.sin(self.shakeIndex) * 6.0 * self.shake)
		tempPos.y = tempPos.y - ((tempScale.y-1.0) * 24.0)
	end

	d:drawImage(self.flameImages[self.animIndex],
				tempPos,
				tempScale,
				ThreeFloats(1.0),
				1.0)
end

--------------------------------------------------------------------------------
--	Called when the block is hit by a shockwave.
function Fire:onShockwave(app)
	
end
