--	Flammable.lua - Class representing a flammable block.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public LFlammablense as published by
--	the Free Software Foundation, either version 3 of the LFlammablense, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public LFlammablense for more details.
--
--	You should have received a copy of the GNU General Public LFlammablense
--	along with this program.  If not, see <http://www.gnu.org/lFlammablenses/>.
--	----------------------------------------------------------------------------

class 'Flammable' (Block)

--------------------------------------------------------------------------------
function Flammable:__init(pos, col)
	Block.__init(self, pos, col)

	self.typeName = "Flammable"

	--If this is less than 1, object will start shaking, then go on fire when it
	--reaches 1.
	self.shake = 1.0
	self.shakeIndex = 0.0
	self.wobbleIndex = 0.0
end

--------------------------------------------------------------------------------
--	Updates the block.
function Flammable:update(app)
	--Update the block's colour.
	if (self.colour ~= self.newColour) then
		self.colIndex = self.colIndex + 0.02

		if (self.colIndex >= 1.0) then
			self.colIndex = 0.0
			self.colour = self.newColour
		end
	else
		--Take on the colour of any neighbours whose colour has changed.
		for i=1,#self.neighbours do
			if (self.neighbours[i].colIndex > 0.5) then
				self.newColour = self.neighbours[i].newColour
			end
		end
	end

	--Shake if we should be shaking.
	if (self.shake < 1.0) then
		if (self.shake < 0.33) and ((self.shake+0.02) > 0.33) then
			particles:triggerSmoke(self.position, ThreeFloats(0.8))
		elseif (self.shake < 0.66) and ((self.shake+0.02) > 0.66) then
			particles:triggerSmoke(self.position, ThreeFloats(0.8))
		end

		self.shake = self.shake + 0.02
		self.shakeIndex = self.shakeIndex + 0.7
		if (self.shakeIndex > (math.pi * 2.0)) then
			self.shakeIndex = 0.0
		end

		self.wobbleIndex = self.wobbleIndex + 0.8
		if (self.wobbleIndex > (math.pi * 2.0)) then
			self.wobbleIndex = 0.0
		end

		if (self.shake >= 1.0) then
			self.canDelete = true
			addFire(self.position)
		end
	end
end

--------------------------------------------------------------------------------
--	Draws the block.
function Flammable:draw(d)
	tempCol = ThreeFloats(self.colour + ((self.newColour-self.colour) * self.colIndex))
	tempPos = TwoFloats(self.position-24.0)
	tempScale = TwoFloats(1.0)

	if (self.shake < 1.0) then
		tempScale.y = 1.0 + (math.sin(self.wobbleIndex) * 0.2 * self.shake)
		tempPos.x = tempPos.x + (math.sin(self.shakeIndex) * 6.0 * self.shake)
		tempPos.y = tempPos.y - ((tempScale.y-1.0) * 24.0)
	end

	d:drawImage(gridImages[5],
				tempPos,
				tempScale,
				tempCol,
				1.0)
end

--------------------------------------------------------------------------------
--	Called when the block is hit by a shockwave.
function Flammable:onShockwave(app)
	if (self.shake >= 1.0) then
		self.shake = 0.0
	end
end
