--	Ice.lua - Class representing an ice block.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

class 'Ice' (Block)

--------------------------------------------------------------------------------
function Ice:__init(pos, col)
	Block.__init(self, pos, col)

	self.colour = hslToRgb(ThreeFloats(0.66, 0.72, 0.93))
	self.newColour = ThreeFloats(self.colour)
	self.typeName = "Ice"

	--If this is less than 1, object will start shaking, then melt when it
	--reaches 1.
	self.shake = 1.0
	self.shakeIndex = 0.0
	self.wobbleIndex = 0.0
end

--------------------------------------------------------------------------------
--	Updates the block.
function Ice:update(app)
	if (self.shake < 1.0) then
		self.shake = self.shake + 0.02
		self.shakeIndex = self.shakeIndex + 0.7
		if (self.shakeIndex > (math.pi * 2.0)) then
			self.shakeIndex = 0.0
		end

		self.wobbleIndex = self.wobbleIndex + 0.8
		if (self.wobbleIndex > (math.pi * 2.0)) then
			self.wobbleIndex = 0.0
		end

		if (self.shake >= 1.0) then
			self.canDelete = true
			addIcePuddles(self.position)
			particles:triggerWater(self.position, self.colour)
		end
	end
end

--------------------------------------------------------------------------------
--	Draws the block.
function Ice:draw(d)
	tempPos = TwoFloats(self.position-24.0)
	tempScale = TwoFloats(1.0)

	if (self.shake < 1.0) then
		tempScale.y = 1.0 + (math.sin(self.wobbleIndex) * 0.2 * self.shake)
		tempPos.x = tempPos.x + (math.sin(self.shakeIndex) * 6.0 * self.shake)
		tempPos.y = tempPos.y - ((tempScale.y-1.0) * 24.0)
	end

	d:drawImage(gridImages[4],
				tempPos,
				tempScale,
				ThreeFloats(1.0),
				1.0)
end

--------------------------------------------------------------------------------
--	Called when the block is hit by a shockwave.
function Ice:onShockwave(app)
	if (self.shake >= 1.0) then
		self.shake = 0.0
	end
end
