--	Pad.lua - Base class for enemy behaviour pads.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

-- Class declaration.
class 'Pad'

--------------------------------------------------------------------------------
--	The class constructor.
function Pad:__init(pos)
	self.position = TwoFloats(pos)
	self.colour = hslToRgb(ThreeFloats(Random.getGlobal():getRandomFloat(), 0.7, 0.8))

	self.image = "WanderPad"
	self.behaviour = behaviourWander

	--Used to ensure pads don't get triggered too often.
	self.timer = 0
end

--------------------------------------------------------------------------------
--	Updates the pad.
function Pad:update(app)
	if (self.timer > 0) then
		self.timer = self.timer - 1
	end
end

--------------------------------------------------------------------------------
--	Draws the pad.
function Pad:draw(d)
	if (self.timer > 0) then
		d:drawImage("PadDown",
					self.position-24.0,
					TwoFloats(1.0),
					self.colour,
					1.0)
	else
		d:drawImage("PadUp",
					self.position-24.0,
					TwoFloats(1.0),
					self.colour,
					1.0)
	end
	if (self.behaviour == behaviourSpeedLeft) then
		d:drawImage(self.image,
				    TwoFloats(self.position.x+24.0, self.position.y-24.0),
				    TwoFloats(-1.0, 1.0),
				    self.colour,
					1.0)
	elseif (self.behaviour == behaviourSpeedUp) then
		d:drawRotImage(self.image,
					   self.position-24.0,
					   -(math.pi*0.5),
					   TwoFloats(24.0),
					   TwoFloats(1.0),
					   self.colour,
					   1.0)
	elseif (self.behaviour == behaviourSpeedDown) then
		d:drawRotImage(self.image,
					   self.position-24.0,
					   (math.pi*0.5),
					   TwoFloats(24.0),
					   TwoFloats(1.0),
					   self.colour,
					   1.0)
	else
		d:drawImage(self.image,
					self.position-24.0,
					TwoFloats(1.0),
					self.colour,
					1.0)
	end
end
