--	Player.lua - The player's class.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

-- Class declaration.
class 'Player'

--------------------------------------------------------------------------------
--	The class constructor.
function Player:__init()
	self.position = TwoFloats(128.0)
	self.colour = hslToRgb(ThreeFloats(Random.getGlobal():getRandomFloat(), 0.7, 0.8))
	self.speed = TwoFloats()
	self.hearts = 3
	self.invincible = 0.0

	self.upIsDown = false
	self.leftIsDown = false
	self.rightIsDown = false
	self.downIsDown = false
	self.joystick = TwoFloats()
end

--------------------------------------------------------------------------------
--	Updates the player.
function Player:update(app)
	--Update invincibility.
	if (self.invincible > 0.0) then
		self.invincible = self.invincible - 0.01
		if (self.invincible < 0.0) then
			self.invincible = 0.0
		end
	end

	--Move the player.
	if self.upIsDown and not self.downIsDown then
		if (self.speed.y > -3.0) then
			self.speed.y = self.speed.y - 0.2
		end
	elseif self.downIsDown and not self.upIsDown then
		if (self.speed.y < 3.0) then
			self.speed.y = self.speed.y + 0.2
		end
	elseif (self.joystick.y > 0.0) and ((self.joystick.y*3.0) > self.speed.y) then
		self.speed.y = self.joystick.y*3.0
	elseif  (self.joystick.y < 0.0) and ((self.joystick.y*3.0) < self.speed.y) then
		self.speed.y = self.joystick.y*3.0
	elseif (self.speed.y < 0.0) then
		self.speed.y = self.speed.y + 0.6
		if (self.speed.y > 0.0) then
			self.speed.y = 0.0
		end
	elseif (self.speed.y > 0.0) then
		self.speed.y = self.speed.y - 0.6
		if (self.speed.y < 0.0) then
			self.speed.y = 0.0
		end
	end

	if self.leftIsDown and not self.rightIsDown then
		if (self.speed.x > -3.0) then
			self.speed.x = self.speed.x - 0.2
		end
	elseif self.rightIsDown and not self.leftIsDown then
		if (self.speed.x < 3.0) then
			self.speed.x = self.speed.x + 0.2
		end
	elseif (self.joystick.x > 0.0) and ((self.joystick.x*3.0) > self.speed.x) then
		self.speed.x = self.joystick.x*3.0
	elseif  (self.joystick.x < 0.0) and ((self.joystick.x*3.0) < self.speed.x) then
		self.speed.x = self.joystick.x*3.0
	elseif (self.speed.x < 0.0) then
		self.speed.x = self.speed.x + 0.6
		if (self.speed.x > 0.0) then
			self.speed.x = 0.0
		end
	elseif (self.speed.x > 0.0) then
		self.speed.x = self.speed.x - 0.6
		if (self.speed.x < 0.0) then
			self.speed.x = 0.0
		end
	end

	if (math.abs(self.speed.x) > 0.0) or (math.abs(self.speed.y) > 0.0) then
		oldPos = TwoFloats(self.position)

		self.position = self.position + self.speed
		tempDir = TwoFloats(self.position - oldPos)
		topLeft = TwoFloats(self.position-20.0)
		bottomRight = TwoFloats(topLeft+40.0)

		for i=0,#blocks do
			if (blocks[i].typeName ~= "Water") and
			   (blocks[i].typeName ~= "Fire") and
			   (blocks[i].typeName ~= "Bomb") then
				blockTL = TwoFloats(blocks[i].position-22.0)
				blockBR = TwoFloats(blockTL+44.0)

				if ((topLeft.x < blockBR.x) and (bottomRight.x > blockTL.x)) and
				   ((topLeft.y < blockBR.y) and (bottomRight.y > blockTL.y)) then
				   --If the x-axis overlap is greater than the y-axis
					if (math.abs(self.position.x-blocks[i].position.x) > math.abs(self.position.y-blocks[i].position.y)) then
						--Overlapping on the left of the block.
						if (topLeft.x < blockTL.x) and (bottomRight.x > blockTL.x) then
							self.position.x = blocks[i].position.x-40.0
						--Overlapping on the right of the block.
						elseif (topLeft.x < blockBR.x) and (bottomRight.x > blockBR.x) then
							self.position.x = blocks[i].position.x+40.0
						end
					else
						--Overlapping on the top of the block.
						if (topLeft.y < blockTL.y) and (bottomRight.y > blockTL.y) then
							self.position.y = blocks[i].position.y-40.0
						--Overlapping on the bottom of the block.
						elseif (topLeft.y < blockBR.y) and (bottomRight.y > blockBR.y) then
							self.position.y = blocks[i].position.y+40.0
						end
					end
				end
			elseif (blocks[i].typeName == "Fire") then -- and (self.invincible <= 0.0) then
				blockTL = TwoFloats(blocks[i].position-22.0)
				blockBR = TwoFloats(blockTL+44.0)

				if ((topLeft.x < blockBR.x) and (bottomRight.x > blockTL.x)) and
				   ((topLeft.y < blockBR.y) and (bottomRight.y > blockTL.y)) then
	
					--Jump the player back.
					--If the x-axis overlap is greater than the y-axis
					if (math.abs(self.position.x-blocks[i].position.x) > math.abs(self.position.y-blocks[i].position.y)) then
						--Overlapping on the left of the block.
						if (topLeft.x < blockTL.x) and (bottomRight.x > blockTL.x) then
							self.position.x = blocks[i].position.x-40.0
						--Overlapping on the right of the block.
						elseif (topLeft.x < blockBR.x) and (bottomRight.x > blockBR.x) then
							self.position.x = blocks[i].position.x+40.0
						end
					else
						--Overlapping on the top of the block.
						if (topLeft.y < blockTL.y) and (bottomRight.y > blockTL.y) then
							self.position.y = blocks[i].position.y-40.0
						--Overlapping on the bottom of the block.
						elseif (topLeft.y < blockBR.y) and (bottomRight.y > blockBR.y) then
							self.position.y = blocks[i].position.y+40.0
						end
					end

					--And hurt them.
					self:hurt(blocks[i].colour)
				end
			end
		end
	end
end

--------------------------------------------------------------------------------
--	Draws the player.
function Player:draw(d)
	alpha = 1.0

	if (self.invincible > 0.0) then
		tempInt = math.floor(self.invincible * 20.0)

		if ((tempInt%2) == 1) then
			alpha = 0.0
		end
	end

	d:drawImage("Player",
				TwoFloats(self.position-24.0),
				TwoFloats(1.0),
				self.colour,
				alpha)
end

--------------------------------------------------------------------------------
--	Moves the player.
function Player:keyDown(key)
	if (key == "CursorUp") then
		self.upIsDown = true
	elseif (key == "CursorDown") then
		self.downIsDown = true
	elseif (key == "CursorLeft") then
		self.leftIsDown = true
	elseif (key == "CursorRight") then
		self.rightIsDown = true
	end
end

--------------------------------------------------------------------------------
--	Moves the player.
function Player:keyUp(key)
	if (key == "CursorUp") then
		self.upIsDown = false
	elseif (key == "CursorDown") then
		self.downIsDown = false
	elseif (key == "CursorLeft") then
		self.leftIsDown = false
	elseif (key == "CursorRight") then
		self.rightIsDown = false
	end
end

--------------------------------------------------------------------------------
--	Moves the player.
function Player:joyMove(axis, value)
	if (axis == 0) or (axis == 2) then
		self.joystick.x = value
	elseif (axis == 1) or (axis == 3) then
		self.joystick.y = value
	end
end

--------------------------------------------------------------------------------
--	Hurts the player.
function Player:hurt(col)
	if (self.invincible <= 0.0) then
		self.hearts = self.hearts - 1
		if (self.hearts > 0) then
			self.invincible = 1.0
		end

		particles:trigger(self.position, col)
		particles:trigger(self.position, col)

		soundsQueue[#soundsQueue+1] = "LoseHeart"
	end
end
