--	Shockwave.lua - Class representing a shockwave.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

-- Class declaration.
class 'Shockwave'

--------------------------------------------------------------------------------
--	The class constructor.
function Shockwave:__init(pos, col)
	self.position = TwoFloats(pos)
	self.colour = ThreeFloats(col)

	self.radius = 0.0
end

--------------------------------------------------------------------------------
--	Updates the shockwave.
function Shockwave:update(app)
	oldRadius = self.radius * 128.0
	oldRadiusSq = oldRadius * oldRadius

	--Update the shockwave's radius.
	if (self.radius < 1.0) then
		self.radius = self.radius + 0.05
	end

	newRadius = self.radius * 128.0
	newRadiusSq = newRadius * newRadius

	--Trigger any nearby blocks.
	for i=0,#blocks do
		distSq = self.position:getDistanceSquared(blocks[i].position)

		if (distSq >= oldRadiusSq) and (distSq < newRadiusSq) then
			blocks[i]:onShockwave(app)
		end
	end

	--Hurt the player if nearby.
	distSq = self.position:getDistanceSquared(player.position)
	if (distSq >= oldRadiusSq) and (distSq < newRadiusSq) then
		player:hurt(self.colour)
	end

	--Hurt any nearby enemies.
	for i=#enemies,1,-1 do
		distSq = self.position:getDistanceSquared(enemies[i].position)
		if (distSq >= oldRadiusSq) and (distSq < newRadiusSq) then
			particles:trigger(enemies[i].position, enemies[i].colour)
			particles:trigger(enemies[i].position, enemies[i].colour)
			table.remove(enemies, i)
		end
	end
end

--------------------------------------------------------------------------------
--	Draws the shockwave.
function Shockwave:draw(d)
	alpha = 1.0

	if (self.radius > 0.75) then
		alpha = 1.0 - ((self.radius-0.75) * 4.0)
	end

	d:drawImage("Shockwave",
				self.position - (128.0 * self.radius * 0.5),
				TwoFloats(self.radius * 0.5),
				self.colour,
				alpha);
end
