--	Water.lua - Class representing a puddle.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

class 'Water' (Block)

--------------------------------------------------------------------------------
function Water:__init(pos, col)
	Block.__init(self, pos, col)

	self.colour = hslToRgb(ThreeFloats(0.66, 0.72, 0.93))
	self.newColour = ThreeFloats(self.colour)
	self.typeName = "Water"

	self.evaporate = false
end

--------------------------------------------------------------------------------
--	Updates the block.
function Water:update(app)
	fireNeighbours = false

	--Check if we're near fire so we can put it out.
	for i=1,#self.neighbours do
		if (self.neighbours[i].typeName == "Fire") then
			if (self.neighbours[i].shake >= 1.0) then
				self.neighbours[i].shake = 0.0
			end

			fireNeighbours = true
			self.evaporate = true
			break
		end
	end

	--If all our fire neighbours have been put out, we can get deleted.
	if self.evaporate and not fireNeighbours then
		self.canDelete = true
	end
end

--------------------------------------------------------------------------------
--	Draws the block.
function Water:draw(d)
	d:drawImage(gridImages[3],
				self.position-24.0,
				TwoFloats(1.0),
				ThreeFloats(1.0),
				1.0)
end

--------------------------------------------------------------------------------
--	Called when the block is hit by a shockwave.
function Water:onShockwave(app)
	
end
