--	LevelGenerator.lua - Functions to generate a level.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

--	Keeps track of the centre of each room.
roomCentres = {}
--	Write index into roomCentres.
centresIndex = 0

--	The player start corner.
playerCorner = 0
--	The player start on the grid.
playerStartPos = TwoFloats()
--	The corner of the stairs.
stairsCorner = 0
--	The stairs position on the grid.
stairsPosGrid = TwoFloats()

--------------------------------------------------------------------------------
--	Alternative level generator.
function generateLevel()
	numRooms = 4 + Random.getGlobal():getRandomInt(3)
	x = 1
	y = 1 + Random.getGlobal():getRandomInt(3)

	for i=#roomCentres,0,-1 do
		table.remove(roomCentres, i)
	end
	centresIndex = 0

	--Top row.
	while (numRooms > 0) do
		roomSizeX = 3 + Random.getGlobal():getRandomInt(4)
		roomSizeY = 3 + Random.getGlobal():getRandomInt(2)

		roomCentres[centresIndex] = TwoFloats(x+(roomSizeX/2), y+(roomSizeY/2))
		centresIndex = centresIndex + 1

		--Add the rooms to the level.
		for j=y,(y+roomSizeY-1) do
			for i=x,(x+roomSizeX-1) do
				if (i > (gridX-2)) then
					break
				end

				grid[j][i] = 101
			end
		end

		--Update x and y.
		x = x + roomSizeX + 2 + Random.getGlobal():getRandomInt(4)
		y = y + (Random.getGlobal():getRandomInt(5) - 2)

		--Make sure they fit in the grid.
		if ((x+3) > (gridX-1)) then
			break
		elseif (y < 1) then
			y = 1
		end

		numRooms = numRooms -1
	end

	--Bottom row.
	numRooms = 4 + Random.getGlobal():getRandomInt(3)
	x = 1 + Random.getGlobal():getRandomInt(4)
	y = math.floor(gridY/2) + Random.getGlobal():getRandomInt(3)

	while (numRooms > 0) do
		roomSizeX = 3 + Random.getGlobal():getRandomInt(4)
		roomSizeY = 3 + Random.getGlobal():getRandomInt(2)

		--Check the room won't overlap any others.
		for j=y,(y+roomSizeY-1) do
			if (j > (gridY-2)) then
				break
			end

			for i=x,(x+roomSizeX-1) do
				if (i > (gridX-2)) then
					break
				end

				if ((grid[j][i] == 101) or
					(grid[j-1][i] == 101) or
					(grid[j-2][i] == 101)) and
				   (j > y) then
					y = y + 2
				end
			end
		end

		roomCentres[centresIndex] = TwoFloats(x+(roomSizeX/2), y+(roomSizeY/2))
		centresIndex = centresIndex + 1

		--Add the rooms to the level.
		for j=y,(y+roomSizeY-1) do
			if (j > (gridY-2)) then
				break
			end

			for i=x,(x+roomSizeX-1) do
				if (i > (gridX-2)) then
					break
				end

				if (j < 0) then
					j = 0
				elseif (i < 0) then
					i = 0
				end

				grid[j][i] = 101
			end
		end

		--Update x and y.
		x = x + roomSizeX + 2 + Random.getGlobal():getRandomInt(3)
		y = y + (Random.getGlobal():getRandomInt(3) - 1)

		--Make sure they fit in the grid.
		if ((x+3) > (gridX-1)) then
			break
		elseif (y < 1) then
			y = 1
		end

		numRooms = numRooms -1
	end
	
	--Add corridors.
	for i=0,(#roomCentres-1) do
		generateCorridor(roomCentres[i], true)
		generateCorridor(roomCentres[i], false)
	end
	generateCorridor(roomCentres[#roomCentres])

	--Convert any rooms to spaces.
	for j=0,gridY-1 do
		for i=0,gridX-1 do
			if (grid[j][i] == 100) or (grid[j][i] == 101) then
				grid[j][i] = 0
			end
		end
	end

	--Add the other block types.
	makeWallsIntoBlocks()
end

--------------------------------------------------------------------------------
--	Generates a corridor that randomly connects existing rooms.
function generateCorridor(start, dir)
	foundStart = false
	x = math.floor(start.x)
	y = math.floor(start.y)
	dirX = 0
	dirY = 0

	if (x < (gridX/2)) then
		dirX = 1
	else
		dirX = -1
	end

	if (y < (gridY/2)) then
		dirY = 1
	else
		dirY = -1
	end

	if dir then
		dirX = 0
	else
		dirY = 0
	end

	--Find the room edge.
	if (x >= gridX) then
		x = gridX-2
	end
	if (y >= gridY) then
		y = gridY-2
	end

	while (grid[y][x] == 101) or 
		  (x > (gridX-2)) or
		  (y > (gridY-2)) do
		if (x > (gridX-2)) then
			x = gridX-2
			dirX = -1
			dirY = 0
		elseif (y > (gridY-2)) then
			y = gridY-2
			dirY = -1
			dirX = 0
		end

		x = x + dirX
		y = y + dirY
	end

	--Start carving out a room.
	failsafe = 32
	turnCount = 7
	while (grid[y][x] ~= 101) and (failsafe > 0) do
		grid[y][x] = 100

		x = x + dirX
		y = y + dirY

		if (x < 1) or (x > (gridX-2)) then
			if (x < 1) then
				x = 1
			elseif (x > (gridX-2)) then
				x = gridX-2
			end

			dirX = 0
			if(y < (gridY/2)) then
				dirY = 1
			else
				dirY = -1
			end
		elseif (y < 1) or (y > (gridY-2)) then
			if (y < 1) then
				y = 1
			elseif (y > (gridY-2)) then
				y = gridY-2
			end

			dirY = 0
			if(x < (gridX/2)) then
				dirX = 1
			else
				dirX = -1
			end
		end

		turnCount = turnCount - 1
		if (turnCount < 0) then
			turnCount = 7

			oldX = dirX
			oldY = dirY

			if (dirX > 0) then
				dirX = 0
				dirY = 1
			elseif (dirX < 0) then
				dirX = 0
				dirY = -1
			elseif (dirY > 0) then
				dirX = -1
				dirY = 0
			elseif (dirY < 0) then
				dirX = 1
				dirY = 0
			end
		end

		failsafe = failsafe - 1
	end
end

--------------------------------------------------------------------------------
--	Turns the walls into wall/ice/flammable/exploding/expando blocks.
function makeWallsIntoBlocks()
	points = {}
	pointBlocks = {}
	pointsIndex = 0
	cellSize = TwoFloats((gridX-1)/5, (gridY-1)/3)

	--Fill out points.
	for j=0,3 do
		for i=0,5 do
			--Set the point's position.
			points[pointsIndex] = TwoFloats(i * cellSize.x,
											j * cellSize.y)

			--Perturb the point's position.
			points[pointsIndex].x = points[pointsIndex].x +
									((Random.getGlobal():getRandomFloat() *
									  cellSize.x) -
									 (cellSize.x*0.5))
			points[pointsIndex].y = points[pointsIndex].y +
									((Random.getGlobal():getRandomFloat() *
									  cellSize.y) -
									 (cellSize.y*0.5))

			--Set the point's block type.
			pointBlocks[pointsIndex] = 1 + Random.getGlobal():getRandomInt(5)
			if (pointBlocks[pointsIndex] > 1) then
				pointBlocks[pointsIndex] = pointBlocks[pointsIndex] + 2
			end

			pointsIndex = pointsIndex + 1
		end
	end

	--Now go through all the walls and set their type accordingly.
	for j=1,(gridY-2) do
		for i=1,(gridX-2) do

			if (grid[j][i] > 0) then
				dist = 10000.0
				nearest = -1

				--Find the nearest point for this wall.
				for k=0,(#points-1) do
					wallPos = TwoFloats(i, j)

					if (wallPos:getDistance(points[k]) < dist) then
						dist = wallPos:getDistance(points[k])
						nearest = k
					end
				end

				--Set the wall to the nearest point's type.
				grid[j][i] = pointBlocks[nearest]
			end
		end
	end
end

--------------------------------------------------------------------------------
--Used to create the correct Blocks from grid.
function createBlocks()
	blockIndex = 0
	blockCols = {}
	tempCol = ThreeFloats(Random.getGlobal():getRandomFloat(), 0.7, 0.8)

	--Setup the colours for the different block types.
	for i=1,4 do
		blockCols[i] = ThreeFloats(hslToRgb(tempCol))
	
		tempCol.x = tempCol.x + 0.2
		if (tempCol.x > 1.0) then
			tempCol.x = tempCol.x - 1.0
		end
	end

	--Make sure blocks is cleared first.
	while (#blocks > 0) do
		table.remove(blocks)
	end

	--Now go through grid, adding the appropriate Blocks.
	for j=0,(gridY-1) do
		for i=0,(gridX-1) do
			tempPos = TwoFloats(gridOffset + 24.0)

			tempPos.x = tempPos.x + (i * 48.0)
			tempPos.y = tempPos.y + (j * 48.0)

			if (grid[j][i] == 1) then
				blocks[blockIndex] = Block(tempPos, blockCols[1])
				blockIndex = blockIndex + 1
			elseif (grid[j][i] == 4) then
				blocks[blockIndex] = Ice(tempPos, blockCols[1])
				blockIndex = blockIndex + 1
			elseif (grid[j][i] == 5) then
				blocks[blockIndex] = Flammable(tempPos, blockCols[2])
				blockIndex = blockIndex + 1
			elseif (grid[j][i] == 6) then
				blocks[blockIndex] = Exploding(tempPos, blockCols[3])
				blockIndex = blockIndex + 1
			elseif (grid[j][i] == 7) then
				blocks[blockIndex] = Expando(tempPos, blockCols[4])
				blockIndex = blockIndex + 1
			end
		end
	end
end

--------------------------------------------------------------------------------
--	Returns the player start position.
function getPlayerStart()
	retval = TwoFloats()
	playerCorner = Random.getGlobal():getRandomInt(4)
	dir = TwoFloats()

	if (playerCorner == 0) then
		retval = TwoFloats(0)
		dir = TwoFloats(1)
		stairsCorner = 3
	elseif (playerCorner == 1) then
		retval = TwoFloats(gridX-1, 0)
		dir = TwoFloats(-1, 1)
		stairsCorner = 2
	elseif (playerCorner == 2) then
		retval = TwoFloats(0, gridY-1)
		dir = TwoFloats(1, -1)
		stairsCorner = 1
	else
		retval = TwoFloats(gridX-1, gridY-1)
		dir = TwoFloats(-1)
		stairsCorner = 0
	end

	while (grid[math.floor(retval.y)][math.floor(retval.x)] ~= 0) do
		retval = retval + dir
	end

	playerStartPos.x = math.floor(retval.x)
	playerStartPos.y = math.floor(retval.y)

	return retval
end

--------------------------------------------------------------------------------
--	Returns the stairs position.
function getStairsPos()
	retval = TwoFloats()
	dir = TwoFloats()

	if (stairsCorner == 0) then
		retval = TwoFloats(0)
		dir = TwoFloats(1)
	elseif (stairsCorner == 1) then
		retval = TwoFloats(gridX-1, 0)
		dir = TwoFloats(-1, 1)
	elseif (stairsCorner == 2) then
		retval = TwoFloats(0, gridY-1)
		dir = TwoFloats(1, -1)
	else
		retval = TwoFloats(gridX-1, gridY-1)
		dir = TwoFloats(-1)
	end

	while (grid[math.floor(retval.y)][math.floor(retval.x)] ~= 0) do
		retval = retval + dir
	end

	stairsPosGrid.x = math.floor(retval.x)
	stairsPosGrid.y = math.floor(retval.y)

	return retval
end

--------------------------------------------------------------------------------
--	Adds 8 enemies to the level.
function addEnemies()
	tempCol = ThreeFloats(0.0, 0.7, 0.8)

	--Get rid of any existing enemies.
	for i=#enemies,1,-1 do
		table.remove(enemies, i)
	end

	--Setup starting positions for each enemy.
	spacing = (gridX-4)/4
	enemyStarts = {TwoFloats(2, 1),
				   TwoFloats(2+spacing, 1),
				   TwoFloats(2+(spacing*2), 1),
				   TwoFloats(2+(spacing*3), 1),
				   TwoFloats(2, gridY-2),
				   TwoFloats(2+spacing, gridY-2),
				   TwoFloats(2+(spacing*2), gridY-2),
				   TwoFloats(2+(spacing*3), gridY-2)}

	spacing2 = (gridX-4-spacing)/4
	if (playerCorner == 0) then
		enemyStarts[1].x = 2+spacing+spacing2
		enemyStarts[2].x = 2+spacing+(spacing2*2)
		enemyStarts[3].x = 2+spacing+(spacing2*3)
		enemyStarts[4].x = 2+spacing+(spacing2*4)
	elseif (playerCorner == 1) then
		enemyStarts[1].x = 2+spacing
		enemyStarts[2].x = 2+spacing+spacing2
		enemyStarts[3].x = 2+spacing+(spacing2*2)
		enemyStarts[4].x = 2+spacing+(spacing2*3)
	elseif (playerCorner == 2) then
		enemyStarts[5].x = 2+spacing+spacing2
		enemyStarts[6].x = 2+spacing+(spacing2*2)
		enemyStarts[7].x = 2+spacing+(spacing2*3)
		enemyStarts[8].x = 2+spacing+(spacing2*4)
	elseif (playerCorner == 3) then
		enemyStarts[5].x = 2+spacing
		enemyStarts[6].x = 2+spacing+spacing2
		enemyStarts[7].x = 2+spacing+(spacing2*2)
		enemyStarts[8].x = 2+spacing+(spacing2*3)
	end

	--Add each enemy
	for i=1,8 do
		tempPos = enemyStarts[i]
		tempPos.x = tempPos.x + Random.getGlobal():getRandomInt(3)

		--Move down until we find an empty space.
		while (grid[math.floor(tempPos.y)][math.floor(tempPos.x)] ~= 0) do
			if (i < 5) then
				tempPos.y = tempPos.y + 1

				if (Random.getGlobal():getRandomFloat() < 0.5) then
					tempPos.y = tempPos.y + 1
				end

				if (tempPos.y >= gridY) then
					if (tempPos.x < (gridX/2)) then
						tempPos.x = tempPos.x + 1
					else
						tempPos.x = tempPos.x - 1
					end
					tempPos.y = 1
				end
			else
				tempPos.y = tempPos.y - 1

				if (Random.getGlobal():getRandomFloat() < 0.5) then
					tempPos.y = tempPos.y - 1
				end

				if (tempPos.y <= 0) then
					if (tempPos.x < (gridX/2)) then
						tempPos.x = tempPos.x + 1
					else
						tempPos.x = tempPos.x - 1
					end
					tempPos.y = gridY-2
				end
			end
		end
		tempPos.x = math.floor(tempPos.x)
		tempPos.y = math.floor(tempPos.y)
		enemies[#enemies+1] = Enemy(TwoFloats(gridOffset+24.0+(tempPos*48.0)),
									hslToRgb(tempCol))

		if (enemies[#enemies].position:getDistance(player.position) < 192.0) then
			
		end

		tempCol.x = tempCol.x + (1.0/8.0)
	end
end

--------------------------------------------------------------------------------
--	Adds a bunch of enemy behaviour pads to the game.
function addPads()
	numPads = 16 + Random.getGlobal():getRandomInt(12)

	--Clear the existing pads.
	for i=#pads,1,-1 do
		table.remove(pads, i)
	end

	while (#pads < numPads) do

		tempPos.x = Random.getGlobal():getRandomInt(gridX)
		tempPos.y = Random.getGlobal():getRandomInt(gridY)

		--Find an empty space.
		while (grid[math.floor(tempPos.y)][math.floor(tempPos.x)] ~= 0) or
			  (tempPos:getDistance(stairsPosGrid) < 2.0) do
			tempPos.x = tempPos.x + 1
			if (tempPos.x >= gridX) then
				tempPos.x = 1

				tempPos.y = tempPos.y + 1
				if (tempPos.y >= gridY) then
					tempPos.y = 1
				end
			end
		end

		--Add the pad.
		pads[#pads+1] = Pad(gridOffset + 24.0 + (tempPos * 48.0))

		--Pick a random pad.
		tempPad = Random.getGlobal():getRandomInt(10)
		if (tempPad == 0) then
			pads[#pads].image = "WanderPad"
			pads[#pads].behaviour = behaviourWander
		elseif (tempPad == 1) then
			pads[#pads].image = "SeekPlayerPad"
			pads[#pads].behaviour = behaviourSeekPlayer
		elseif (tempPad == 2) then
			pads[#pads].image = "SpeedPad"
			pads[#pads].behaviour = behaviourSpeedRight
			if (grid[math.floor(tempPos.y)][math.floor(tempPos.x)+1] ~= 0) then
				table.remove(pads, #pads)
			end
		elseif (tempPad == 3) then
			pads[#pads].image = "SpeedPad"
			pads[#pads].behaviour = behaviourSpeedLeft
			if (grid[math.floor(tempPos.y)][math.floor(tempPos.x)-1] ~= 0) then
				table.remove(pads, #pads)
			end
		elseif (tempPad == 4) then
			pads[#pads].image = "SpeedPad"
			pads[#pads].behaviour = behaviourSpeedUp
			if (grid[math.floor(tempPos.y)-1][math.floor(tempPos.x)] ~= 0) then
				table.remove(pads, #pads)
			end
		elseif (tempPad == 5) then
			pads[#pads].image = "SpeedPad"
			pads[#pads].behaviour = behaviourSpeedDown
			if (grid[math.floor(tempPos.y)+1][math.floor(tempPos.x)] ~= 0) then
				table.remove(pads, #pads)
			end
		elseif (tempPad == 6) then
			pads[#pads].image = "CirclePad"
			pads[#pads].behaviour = behaviourCircle
		elseif (tempPad == 7) then
			pads[#pads].image = "BombPad"
			pads[#pads].behaviour = behaviourBomb
		elseif (tempPad == 8) then
			pads[#pads].image = "FlamePad"
			pads[#pads].behaviour = behaviourFlame
		elseif (tempPad == 9) then
			pads[#pads].image = "WaterPad"
			pads[#pads].behaviour = behaviourWater
		end
	end
end
